@@ -419,7 +419,7 @@ class Parser:
             self.expr_stmt_1     = self.expr_stmt_1__26
             self.yield_expr      = self.yield_expr__26
             return
-        elif version in ((3, 0), (3, 1), (3, 2), (3, 3), (3, 4), (3, 5)):
+        elif version in ((3, 0), (3, 1), (3, 2), (3, 3), (3, 4), (3, 5), (3, 6)):
             if version == (3, 0):
                 self.with_stmt       = self.with_stmt__26 # lol
             else:
@@ -520,6 +520,8 @@ class Parser:
     def add_flags(self, flags):
         if "print_function" in flags:
             self.lexer.print_function = True
+        if "unicode_literals" in flags:
+            self.lexer.unicode_literals = True
 
     # Grammar
     @action(Expect(Alt(Newline(),
@@ -546,17 +548,26 @@ class Parser:
         """eval_input: testlist NEWLINE* ENDMARKER"""
         return ast.Expression(body=[expr], loc=expr.loc)
 
-    @action(Seq(Loc("@"), Rule("dotted_name"),
+    @action(Seq(Loc("@"), List(Tok("ident"), ".", trailing=False),
                 Opt(BeginEnd("(", Opt(Rule("arglist")), ")",
                              empty=_empty_arglist)),
                 Loc("newline")))
-    def decorator(self, at_loc, dotted_name, call_opt, newline_loc):
+    def decorator(self, at_loc, idents, call_opt, newline_loc):
         """decorator: '@' dotted_name [ '(' [arglist] ')' ] NEWLINE"""
-        name_loc, name = dotted_name
-        expr = ast.Name(id=name, ctx=None, loc=name_loc)
+        root = idents[0]
+        dec_loc = root.loc
+        expr = ast.Name(id=root.value, ctx=None, loc=root.loc)
+        for ident in idents[1:]:
+          dot_loc = ident.loc.begin()
+          dot_loc.begin_pos -= 1
+          dec_loc = dec_loc.join(ident.loc)
+          expr = ast.Attribute(value=expr, attr=ident.value, ctx=None,
+                               loc=expr.loc.join(ident.loc),
+                               attr_loc=ident.loc, dot_loc=dot_loc)
+
         if call_opt:
             call_opt.func = expr
-            call_opt.loc = name_loc.join(call_opt.loc)
+            call_opt.loc = dec_loc.join(call_opt.loc)
             expr = call_opt
         return at_loc, expr
 
@@ -894,7 +905,12 @@ class Parser:
         return ast.Print(dest=dest, values=values, nl=nl,
                          dest_loc=dest_loc, loc=loc)
 
-    @action(Seq(Loc("print"), Alt(print_stmt_1, print_stmt_2)))
+    @action(Eps())
+    def print_stmt_3(self, eps):
+        return ast.Print(dest=None, values=[], nl=True,
+                         dest_loc=None, loc=None)
+
+    @action(Seq(Loc("print"), Alt(print_stmt_1, print_stmt_2, print_stmt_3)))
     def print_stmt(self, print_loc, stmt):
         """
         (2.6-2.7)
@@ -902,7 +918,10 @@ class Parser:
                               '>>' test [ (',' test)+ [','] ] )
         """
         stmt.keyword_loc = print_loc
-        stmt.loc = print_loc.join(stmt.loc)
+        if stmt.loc is None:
+            stmt.loc = print_loc
+        else:
+            stmt.loc = print_loc.join(stmt.loc)
         return stmt
 
     @action(Seq(Loc("del"), List(Rule("expr"), ",", trailing=True)))
@@ -1148,7 +1167,7 @@ class Parser:
         if else_opt:
             stmt.else_loc, stmt.else_colon_loc, stmt.orelse = else_opt
 
-        for elif_ in elifs:
+        for elif_ in reversed(elifs):
             stmt.keyword_loc, stmt.test, stmt.if_colon_loc, stmt.body = elif_
             stmt.loc = stmt.keyword_loc.join(stmt.body[-1].loc)
             if stmt.orelse:
@@ -1493,7 +1512,7 @@ class Parser:
     def atom_2(self, tok):
         return ast.Name(id=tok.value, loc=tok.loc, ctx=None)
 
-    @action(Alt(Tok("int"), Tok("float"), Tok("complex")))
+    @action(Alt(Tok("int"), Tok("float"), Tok("complex"), Tok("long")))
     def atom_3(self, tok):
         return ast.Num(n=tok.value, loc=tok.loc)
 
@@ -1505,7 +1524,10 @@ class Parser:
 
     @action(Plus(atom_4))
     def atom_5(self, strings):
-        return ast.Str(s="".join([x.s for x in strings]),
+        joint = ""
+        if all(isinstance(x.s, bytes) for x in strings):
+            joint = b""
+        return ast.Str(s=joint.join([x.s for x in strings]),
                        begin_loc=strings[0].begin_loc, end_loc=strings[-1].end_loc,
                        loc=strings[0].loc.join(strings[-1].loc))
 
@@ -1788,13 +1810,9 @@ class Parser:
                             name_loc=name_tok.loc, colon_loc=colon_loc,
                             loc=class_loc.join(body[-1].loc))
 
-    @action(Rule("argument"))
-    def arglist_1(self, arg):
-        return [arg], self._empty_arglist()
-
     @action(Seq(Loc("*"), Rule("test"), Star(SeqN(1, Tok(","), Rule("argument"))),
                 Opt(Seq(Tok(","), Loc("**"), Rule("test")))))
-    def arglist_2(self, star_loc, stararg, postargs, kwarg_opt):
+    def arglist_1(self, star_loc, stararg, postargs, kwarg_opt):
         dstar_loc = kwarg = None
         if kwarg_opt:
             _, dstar_loc, kwarg = kwarg_opt
@@ -1811,31 +1829,32 @@ class Parser:
                         star_loc=star_loc, dstar_loc=dstar_loc, loc=None)
 
     @action(Seq(Loc("**"), Rule("test")))
-    def arglist_3(self, dstar_loc, kwarg):
+    def arglist_2(self, dstar_loc, kwarg):
         return [], \
                ast.Call(args=[], keywords=[], starargs=None, kwargs=kwarg,
                         star_loc=None, dstar_loc=dstar_loc, loc=None)
 
-    @action(SeqN(0, Rule("argument"), Tok(",")))
-    def arglist_4(self, arg):
-        return [], ([arg], self._empty_arglist())
+    @action(Seq(Rule("argument"),
+                Alt(SeqN(1, Tok(","), Alt(Rule("arglist_1"),
+                                          Rule("arglist_2"),
+                                          Rule("arglist_3"),
+                                          Eps())),
+                    Eps())))
+    def arglist_3(self, arg, cont):
+        if cont is None:
+            return [arg], self._empty_arglist()
+        else:
+            args, rest = cont
+            return [arg] + args, rest
 
-    @action(Alt(Seq(Star(SeqN(0, Rule("argument"), Tok(","))),
-                    Alt(arglist_1, arglist_2, arglist_3)),
-                arglist_4))
-    def arglist(self, pre_args, rest):
-        # Python's grammar is very awkwardly formulated here in a way
-        # that is not easily amenable to our combinator approach.
-        # Thus it is changed to the equivalent:
-        #
-        #     arglist: (argument ',')* ( argument | ... ) | argument ','
-        #
+    @action(Alt(Rule("arglist_1"),
+                Rule("arglist_2"),
+                Rule("arglist_3")))
+    def arglist(self, args, call):
         """arglist: (argument ',')* (argument [','] |
                                      '*' test (',' argument)* [',' '**' test] |
                                      '**' test)"""
-        post_args, call = rest
-
-        for arg in pre_args + post_args:
+        for arg in args:
             if isinstance(arg, ast.keyword):
                 call.keywords.append(arg)
             elif len(call.keywords) > 0:
